/*
 * PhoneApplet.java
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.net.URL;
import java.util.Properties;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.IOException; 
import java.applet.Applet;
import java.awt.*;


/**
 * Simple applet to display the results from the phone servlet.
 */
public class PhoneApplet extends Applet {

    /**
     * Lay out the UI components.
     */
    public void init() {
	setLayout(new BorderLayout());

	Panel headerPanel = new Panel();
	headerPanel.setLayout(new BorderLayout());

        Label title = new Label("Call the Phone Servlet");
        title.setFont(new Font("Helvetica", Font.BOLD, 18));
	Panel titlePanel = new Panel();
	titlePanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	titlePanel.add(title);

	headerPanel.add("North", titlePanel);

	Label nameLabel = new Label("Find name:");
	nameField = new TextField(20);
	Panel namePanel = new Panel();
	namePanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	namePanel.add(nameLabel);
	namePanel.add(nameField);
	headerPanel.add("South", namePanel);

	Panel bp = new Panel();
	bp.setLayout(new FlowLayout(FlowLayout.CENTER));
	bp.add(searchButton = new Button("Search"));
	
        add("North", headerPanel);
        add("Center", textArea = new TextArea(10, 20));
        add("South", bp);
    }


    /**
     * Call the phone servlet with no arguments and display its results.
     */
    public void start() {
	callServlet();
    }


    /**
     * Call the servlet when the Search button is clicked or Return is
     * pressed in the name field.
     */
    public boolean action(Event evt, Object arg) {
	if (evt.target instanceof TextField || evt.target instanceof Button) {
	    callServlet();
	    return true;
	}
	return false;
    }


    /**
     * Call the phone servlet with name found in the name field (if any).
     */
    private void callServlet() {
        try {
            URL phoneServlet = new URL(getDocumentBase(), "servlet/phone");
            ServletMessage phone = new ServletMessage(phoneServlet);
            
            Properties args = new Properties();
	    String name = nameField.getText();
	    if (name != null && name.length() > 0) {
		args.put("name", name);   // One of the names in the phone list
	    }
            InputStream result = phone.sendMessage(args, ServletMessage.POST);

            DataInputStream in = new DataInputStream(result);
	    textArea.setText("");
            String line;
            while ((line = in.readLine()) != null) {
                textArea.appendText(line + "\n");
            }
        } catch (IOException exc) {
            showStatus("Cannot read phone list: " + exc.getMessage());
        }
    }


    // Private members
    private TextArea textArea;
    private TextField nameField;
    private Button searchButton;
}
