/*
 * @(#)ServletMessage.java	1.3 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package com.sun.server.calendar;

import java.util.Hashtable;
import java.util.Enumeration;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.io.*;
import com.sun.server.admin.security.*;

/**
 * The protocol is simply a URLEncoded string to the servlet with
 * category and method names is the QueryString set to tell the
 * server what action is to be performed. The rest of the name
 * value pairs are arguments to that method on the servlet.
 */

public class ServletMessage {

    public static final int GET  = 0;
    public static final int POST = 1;

    URL servlet;

    /**
     * @param servlet the servlet to send messages to 
     */
    public ServletMessage(URL servlet) {
	this.servlet = servlet;
    }


    /**
     * @return the URLEncoded string representing this message
     */
    public String toEncodedString(Hashtable h) {
	StringBuffer sb = new StringBuffer();
	if (h != null) {
	    String key;
	    for (Enumeration e = h.keys(); e.hasMoreElements();) {
		key = (String)e.nextElement();
		sb.append("&" + URLEncoder.encode(key) + "=" +
			  URLEncoder.encode((String)h.get(key)));
	    }
	}
	return sb.toString();
    }
 
    /**
     * @return the response from the servlet to this message 
     */
    public ServletResponse sendMessage(Hashtable args, int method) { 
	ServletResponse res = null;
	URL url;

	try {
	    if (method == GET) {
		url = new URL(servlet + "?" + toEncodedString(args)); 
		res = new ServletResponse(url.openStream()); 
	    } else if (method == POST) {
		url = servlet;
		URLConnection conn = url.openConnection();
		conn.setDoOutput(true);

		// Work around Netscape's setting of multipart-mime for
		// all POSTs.
		conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		PrintStream ps = new PrintStream(conn.getOutputStream());
		ps.print(toEncodedString(args));
		ps.flush();
		ps.close();
		res = new ServletResponse(conn.getInputStream());
	    }
	} catch (IOException e) {
	    e.printStackTrace();
	    res = null;
	}
	return res;
    }   
}

