/*
 * @(#)WeeklyCanvas.java	1.5 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

package com.sun.server.calendar;

import java.awt.*;
import java.util.*;

/**
 * WeeklyCanvas
 * 
 * @author   Jongyoon Lee
 * @version  1.5, 03/03/97
 */


public class WeeklyCanvas extends Canvas {

    final int ORIGINX = 40;
    final int ORIGINY = 30;
    final int COLUMNWIDTH  = 90;
    final int COLUMNHEIGHT = 25;
    static Hashtable days = new Hashtable();

    public int clickedColumn = 0;
    public Schedule schedSelected = null;
    public Vector[] schedules;

    static {
	days.put("0", "Sun");
	days.put("1", "Mon");
	days.put("2", "Tue");
	days.put("3", "Wed");
	days.put("4", "Thu");
	days.put("5", "Fri");
	days.put("6", "Sat");
    }

    private int x, y;
    private Date[] dates;

    public WeeklyCanvas() {
	super();
	x = 0; y = 0;
	dates = new Date[5];
	schedules = new Vector[5];
    }

    public void selectSchedule(String header, Date when) {
	int columnNum = when.getDay() - 1;
	Vector v = schedules[columnNum];
	if (v == null) {
	    return;
	}
	Enumeration e = v.elements();
	Schedule s;
	String buf;
	while (e.hasMoreElements()) {
	    s = (Schedule) e.nextElement();
	    buf = s.message;
	    buf = (buf.indexOf('\n') != -1) ?
		buf.substring(0, buf.indexOf('\n')) :
		buf;
	    if (buf.equals(header)) {
		schedSelected = s;
		return;
	    }
	}
	System.out.println("No such schedule!");
	schedSelected = null;
    }


    public void setColumn(Vector schedule, Date header) {
	int columnNum = header.getDay() - 1;
	dates[columnNum] = header;
	schedules[columnNum] = schedule;
    }


    public void redrawColumn(int columnNum) {
	Vector schedule = schedules[columnNum];
	Date header = dates[columnNum];
	Graphics gc = getGraphics();
	int oy = ORIGINY;
	int ox = ORIGINX + columnNum * COLUMNWIDTH;

	String date = (String) days.get(String.valueOf(header.getDay()));
	date = date + " " + String.valueOf(header.getDate());
	FontMetrics fm = gc.getFontMetrics();
	gc.drawString(date, ox +
		      (COLUMNWIDTH - fm.stringWidth(date))/2, oy - 2);
	for (int i = 0; i < 20; i+=2) {
	    gc.setColor(Color.white);
	    gc.drawRect(ox-1, oy-1, COLUMNWIDTH, COLUMNHEIGHT*2);
	    gc.setColor(Color.darkGray);
	    gc.drawRect(ox, oy, COLUMNWIDTH, COLUMNHEIGHT*2);
	    gc.setColor(Color.gray);
	    gc.drawLine(ox, oy+COLUMNHEIGHT, ox+COLUMNWIDTH, oy+COLUMNHEIGHT);
	    oy += COLUMNHEIGHT + COLUMNHEIGHT;
	}

	if (schedule == null) {
	    return;
	}

	Enumeration e = schedule.elements();
	Font ft = gc.getFont();
	gc.setFont(new Font("Courier", Font.PLAIN, 8));
	String msg;
	while (e.hasMoreElements()) {
	    Schedule sched = (Schedule) e.nextElement();
	    ox = ORIGINX + columnNum * COLUMNWIDTH + 1;
	    oy = ORIGINY + 1;
	    gc.setColor(Color.blue);
	    gc.fillRect(ox, oy + COLUMNHEIGHT * sched.start,
			COLUMNWIDTH - 2,
			(sched.end - sched.start) * COLUMNHEIGHT - 2);
	    gc.setColor(Color.white);
	    msg = (sched.message.indexOf('\n') != -1) ?
		sched.message.substring(0, sched.message.indexOf('\n')) :
		sched.message;
	    msg = (msg.length() > 16) ? msg.substring(0, 13) + "..." : msg;
	    gc.drawString(msg, ox + 2, oy + COLUMNHEIGHT * (sched.start+1) - 3);
	}
	gc.setFont(ft);
	gc.setColor(Color.black);
    }


    private void redrawTime(Graphics gc) {
	int ox = ORIGINX - 2;
	int oy = ORIGINY + 10;
	String str;
	FontMetrics fm = gc.getFontMetrics();
	for (int i = 8; i < 18; i++) {
	    str = String.valueOf((i > 12) ? (i - 12) : i);
	    gc.drawString(str, ox - fm.stringWidth(str), oy);
	    oy += COLUMNHEIGHT + COLUMNHEIGHT;
	}
    }


    private void findGrid() {
	schedSelected = null;

	x -= ORIGINX;
	y -= ORIGINY;

	if (x < 0 || y < 0) {
	    return;
	}

	x = x / COLUMNWIDTH;
	y = y / COLUMNHEIGHT;

	clickedColumn = x;

	if (schedules[x] == null) {
	    return;
	}

	Enumeration e = schedules[x].elements();
	while (e.hasMoreElements()) {
	    Schedule s = (Schedule) e.nextElement();
	    if (s.start <= y && y < s.end) {
		schedSelected = s;
		return;
	    }
	}
    }


    public void paint(Graphics gc) {
	redrawTime(gc);
	for (int i = 0; i < 5; i++) {
	    if (dates[i] != null) {
		redrawColumn(i);
	    }
	}
    }


    public boolean handleEvent(Event e) {
	switch (e.id) {
	case Event.MOUSE_DOWN:
	    this.x = e.x;
	    this.y = e.y;
	    findGrid();
	    repaint();
	    return true;
	}
	return super.handleEvent(e);
    }

    public Dimension preferredSize() {
	return new Dimension(500, 580);
    }

    public Dimension minimumSize() {
	return preferredSize();
    }
}

class Schedule {
    public int start;
    public int end;
    public String requestor;
    public String attendees;
    public String message;
}
